package org.springframework.web.jsf;

import javax.faces.application.Application;
import javax.faces.application.ApplicationFactory;

/**
 * Replaces original ApplicationFactory's Application with a SpringApplication
 * that wraps the orginal Application.
 * <p>
 * @author Ken Weiner
 * @see SpringApplication
 */
public class SpringApplicationFactory extends ApplicationFactory {
    
    private ApplicationFactory originalApplicationFactory;

    public SpringApplicationFactory() {
        // Default constructor
    }

    /**
     * Replaces the original ApplicationFactory's Application
     * with the SpringApplication.
     * @param applicationFactory the original ApplicationFactory
     */
    public SpringApplicationFactory(ApplicationFactory applicationFactory) {
        this.originalApplicationFactory = applicationFactory;
        Application originalApplication = applicationFactory.getApplication();
        setApplication(originalApplication);
    }

    /**
     * Delegates the retrieval of the Application to the
     * original ApplicationFactory.
     * @return the Application from the original application factory
     */
    public Application getApplication() {
        return this.originalApplicationFactory.getApplication();
    }

    /**
     * Sets the Application in the original ApplicationFactory
     * after wrapping it in a SpringApplication if it isn't already
     * an instance of one.
     */
    public void setApplication(Application application) {
        if (!(application instanceof SpringApplication)) {
            Application springApplication = new SpringApplication(application);
            this.originalApplicationFactory.setApplication(springApplication);
        } else {
            this.originalApplicationFactory.setApplication(application);
        }
    }
}