package org.springframework.web.jsf;

import java.beans.FeatureDescriptor;
import java.util.Iterator;
import java.util.Map;

import javax.el.ELContext;
import javax.el.ELResolver;
import javax.faces.context.FacesContext;

import org.springframework.context.ApplicationContext;

/**
 * adds all ELResolvers defined in spring's root context. 
 * @author Stefan Rinke
 */
public class SpringELResolverSupport extends ELResolver {
	
	Map<ELResolver,ELResolver> resolvers = null;

	@SuppressWarnings("unchecked")
	private void initResolvers(ELContext ctx) {
		FacesContext context = (FacesContext) ctx.getContext(FacesContext.class);
		if( context != null ) {
			ApplicationContext appCtx = FacesContextUtils.getRequiredWebApplicationContext(context);
			resolvers = appCtx.getBeansOfType(ELResolver.class);
		}
	}
	
	@Override
	public Class<?> getCommonPropertyType(ELContext ctx, Object base) {
		if( resolvers == null ) initResolvers(ctx);
		return Object.class;
	}


	@Override
	public Iterator<FeatureDescriptor> getFeatureDescriptors(ELContext arg0, Object arg1) {
		return null;
	}

	@Override
	public Class<?> getType(ELContext ctx, Object base, Object prop) {
		if( resolvers == null ) initResolvers(ctx);
		for (ELResolver resolver : resolvers.keySet()) {
			Class<?> clz = resolver.getType(ctx, base, prop);
			if( ctx.isPropertyResolved() ) return clz;
		}
		return null;
	}

	@Override
	public Object getValue(ELContext ctx, Object base, Object prop) {
		if( resolvers == null ) initResolvers(ctx);
		for (ELResolver resolver : resolvers.keySet()) {
			Object value = resolver.getValue(ctx, base, prop);
			if( ctx.isPropertyResolved() ) return value;
		}
		return null;
	}

	@Override
	public boolean isReadOnly(ELContext ctx, Object base, Object prop) {
		if( resolvers == null ) initResolvers(ctx);
		for (ELResolver resolver : resolvers.keySet()) {
			boolean readOnly = resolver.isReadOnly(ctx, base, prop);
			if( ctx.isPropertyResolved() ) return readOnly;
		}
		return false;
	}

	@Override
	public void setValue(ELContext ctx, Object base, Object prop, Object value) {
		if( resolvers == null ) initResolvers(ctx);
		for (ELResolver resolver : resolvers.keySet()) {
			resolver.setValue(ctx, base, prop, value);
			if( ctx.isPropertyResolved() ) break;
		}
	}

}
